@@ -1,5 +1,5 @@
 // build/run probes
-// Copyright (C) 2005-2010 Red Hat Inc.
+// Copyright (C) 2005-2011 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -34,7 +34,8 @@ static int uprobes_pass (systemtap_session& s);
 
 /* Adjust and run make_cmd to build a kernel module. */
 static int
-run_make_cmd(systemtap_session& s, string& make_cmd)
+run_make_cmd(systemtap_session& s, vector<string>& make_cmd,
+             bool null_out=false, bool null_err=false)
 {
   // Before running make, fix up the environment a bit.  PATH should
   // already be overridden.  Clean out a few variables that
@@ -46,6 +47,7 @@ run_make_cmd(systemtap_session& s, string& make_cmd)
     {
       const char* e = strerror (errno);
       cerr << "unsetenv failed: " << e << endl;
+      s.set_try_server ();
     }
 
   // Disable ccache to avoid saving files that will never be reused.
@@ -55,11 +57,14 @@ run_make_cmd(systemtap_session& s, string& make_cmd)
   (void) setenv("CCACHE_DISABLE", "1", 0);
 
   if (s.verbose > 2)
-    make_cmd += " V=1";
+    make_cmd.push_back("V=1");
   else if (s.verbose > 1)
-    make_cmd += " --no-print-directory";
+    make_cmd.push_back("--no-print-directory");
   else
-    make_cmd += " -s --no-print-directory";
+    {
+      make_cmd.push_back("-s");
+      make_cmd.push_back("--no-print-directory");
+    }
 
   // NB: there appears to be no parallelism opportunity in the
   // module-building makefiles, so while the following works, it
@@ -67,7 +72,7 @@ run_make_cmd(systemtap_session& s, string& make_cmd)
 #if 0
   long smp = sysconf(_SC_NPROCESSORS_ONLN);
   if (smp > 1)
-    make_cmd += " -j " + lex_cast(smp);
+    make_cmd.push_back("-j" + lex_cast(smp));
 #endif
 
   if (strverscmp (s.kernel_base_release.c_str(), "2.6.29") < 0)
@@ -75,10 +80,13 @@ run_make_cmd(systemtap_session& s, string& make_cmd)
       // Older kernels, before linux commit #fd54f502841c1, include
       // gratuitous "echo"s in their Makefile.  We need to suppress
       // that with this bluntness.
-      make_cmd += " >/dev/null";
+      null_out = true;
     }
 
-  return stap_system (s.verbose, make_cmd);
+  rc = stap_system (s.verbose, make_cmd, null_out, null_err);
+  if (rc != 0)
+    s.set_try_server ();
+  return rc;
 }
 
 static void
@@ -117,7 +125,10 @@ compile_pass (systemtap_session& s)
 {
   int rc = uprobes_pass (s);
   if (rc)
-    return rc;
+    {
+      s.set_try_server ();
+      return rc;
+    }
 
   // fill in a quick Makefile
   string makefile_nm = s.tmpdir + "/Makefile";
@@ -207,11 +218,13 @@ compile_pass (systemtap_session& s)
   output_autoconf(s, o, "autoconf-mm-context-vdso.c", "STAPCONF_MM_CONTEXT_VDSO", NULL);
   output_autoconf(s, o, "autoconf-blk-types.c", "STAPCONF_BLK_TYPES", NULL);
   output_autoconf(s, o, "autoconf-perf-structpid.c", "STAPCONF_PERF_STRUCTPID", NULL);
+  output_autoconf(s, o, "autoconf-kern-path-parent.c",
+		  "STAPCONF_KERN_PATH_PARENT", NULL);
 
   o << module_cflags << " += -include $(STAPCONF_HEADER)" << endl;
 
   for (unsigned i=0; i<s.macros.size(); i++)
-    o << "EXTRA_CFLAGS += -D " << lex_cast_qstring(s.macros[i]) << endl;
+    o << "EXTRA_CFLAGS += -D " << lex_cast_qstring(s.macros[i]) << endl; // XXX right quoting?
 
   if (s.verbose > 3)
     o << "EXTRA_CFLAGS += -ftime-report -Q" << endl;
@@ -258,27 +271,32 @@ compile_pass (systemtap_session& s)
   rc = stat(module_dir_makefile.c_str(), &st);
   if (rc != 0)
     {
-	clog << "Checking \"" << module_dir_makefile << "\" failed: " << strerror(errno) << endl
-	     << "Ensure kernel development headers & makefiles are installed." << endl;
+        clog << _F("Checking \" %s \" failed with error: %s\nEnsure kernel development headers & makefiles are installed.",
+                   module_dir_makefile.c_str(), strerror(errno)) << endl;
+	s.set_try_server ();
 	return rc;
     }
 
   // Run make
-  string make_cmd = string("make")
-    + string (" -C \"") + module_dir + string("\""); // XXX: lex_cast_qstring?
-  make_cmd += string(" M=\"") + s.tmpdir + string("\"");
+  vector<string> make_cmd;
+  make_cmd.push_back("make");
+  make_cmd.push_back("-C");
+  make_cmd.push_back(module_dir);
+  make_cmd.push_back("M=" + s.tmpdir); // need make-quoting?
 
-  // Add architecture
-  make_cmd += string(" ARCH=") + lex_cast_qstring(s.architecture);
+  // Add architecture, except for old powerpc (RHBZ669082)
+  if (s.architecture != "powerpc" ||
+      (strverscmp (s.kernel_base_release.c_str(), "2.6.15") >= 0))
+    make_cmd.push_back("ARCH=" + s.architecture); // need make-quoting?
 
   // Add any custom kbuild flags
-  for (unsigned k=0; k<s.kbuildflags.size(); k++)
-    make_cmd += string(" ") + lex_cast_qstring(s.kbuildflags[k]);
+  make_cmd.insert(make_cmd.end(), s.kbuildflags.begin(), s.kbuildflags.end());
 
-  make_cmd += string (" modules");
+  make_cmd.push_back("modules");
 
   rc = run_make_cmd(s, make_cmd);
-
+  if (rc)
+    s.set_try_server ();
   return rc;
 }
 
@@ -290,8 +308,11 @@ compile_pass (systemtap_session& s)
 static bool
 kernel_built_uprobes (systemtap_session& s)
 {
-  string grep_cmd = string ("/bin/grep -q unregister_uprobe ") + 
-    s.kernel_build_tree + string ("/Module.symvers");
+  vector<string> grep_cmd;
+  grep_cmd.push_back ("/bin/grep");
+  grep_cmd.push_back ("-q");
+  grep_cmd.push_back ("unregister_uprobe");
+  grep_cmd.push_back (s.kernel_build_tree + "/Module.symvers");
 
   return (stap_system (s.verbose, grep_cmd) == 0);
 }
@@ -313,7 +334,7 @@ may_build_uprobes (const systemtap_session& s)
   string uprobes_home = s.runtime_path + "/uprobes";
   struct stat file_info;
   if (stat(uprobes_home.c_str(), &file_info) != 0) {
-    clog << "Unable to obtain information on " << uprobes_home << '.' << endl;
+    clog << _("Unable to obtain information on ") << uprobes_home << '.' << endl;
     return false;
   }
 
@@ -336,32 +357,35 @@ static bool
 verify_uprobes_uptodate (systemtap_session& s)
 {
   if (s.verbose > 1)
-    clog << "Pass 4, preamble: "
-	 << "verifying that SystemTap's version of uprobes is up to date."
+    clog << _("Pass 4, preamble: verifying that SystemTap's version of uprobes is up to date.")
 	 << endl;
 
   string uprobes_home = s.runtime_path + "/uprobes";
-  string make_cmd = string("make -q -C ") + uprobes_home
-    + string(" uprobes.ko");
+  vector<string> make_cmd;
+  make_cmd.push_back("make");
+  make_cmd.push_back("-q");
+  make_cmd.push_back("-C");
+  make_cmd.push_back(uprobes_home);
+  make_cmd.push_back("uprobes.ko");
   int rc = run_make_cmd(s, make_cmd);
   if (rc) {
-    clog << "SystemTap's version of uprobes is out of date." << endl;
+    clog << _("SystemTap's version of uprobes is out of date.") << endl;
 
     struct stat file_info;
     if (stat(uprobes_home.c_str(), &file_info) != 0) {
-      clog << "Unable to obtain information on " << uprobes_home << '.' << endl;
+      clog << _("Unable to obtain information on ") << uprobes_home << '.' << endl;
     }
     else {
       struct passwd *owner = getpwuid (file_info.st_uid);
-      string owner_name = owner == NULL ? "The owner of " + uprobes_home :
+      string owner_name = owner == NULL ? _("The owner of ") + uprobes_home :
 	                                  owner->pw_name;
       if (owner_name == "root")
 	owner_name = "";
       struct group *owner_group = getgrgid (file_info.st_gid);
-      string owner_group_name = owner_group == NULL ? "The owner group of " + uprobes_home :
+      string owner_group_name = owner_group == NULL ? _("The owner group of ") + uprobes_home :
 	                                              owner_group->gr_name;
-      clog << "As root, " << owner_name << (owner_name.empty () ? "" : ", ")
-	   << "or a member of the '" << owner_group_name << "' group, run" << endl;
+      clog << _F("As root, %s%s or a member of the '%s' group, run\n",
+                 owner_name.c_str(), (owner_name.empty() ? "" : ", "), owner_group_name.c_str());
       clog << "\"make -C " << uprobes_home << "\"." << endl;
     }
   }
@@ -373,16 +397,19 @@ static int
 make_uprobes (systemtap_session& s)
 {
   if (s.verbose > 1)
-    clog << "Pass 4, preamble: "
-	 << "(re)building SystemTap's version of uprobes."
+    clog << _("Pass 4, preamble: (re)building SystemTap's version of uprobes.")
 	 << endl;
 
   string uprobes_home = s.runtime_path + "/uprobes";
-  string make_cmd = string("make -C ") + uprobes_home;
+  vector<string> make_cmd;
+  make_cmd.push_back("make");
+  make_cmd.push_back("-C");
+  make_cmd.push_back(uprobes_home);
   int rc = run_make_cmd(s, make_cmd);
   if (s.verbose > 1)
-    clog << "uprobes rebuild rc=" << rc << endl;
-
+    clog << _("uprobes rebuild exit code: ") << rc << endl;
+  if (rc)
+    s.set_try_server ();
   return rc;
 }
 
@@ -393,9 +420,10 @@ make_uprobes (systemtap_session& s)
 static int
 copy_uprobes_symbols (const systemtap_session& s)
 {
-  string uprobes_home = s.runtime_path + "/uprobes";
-  string cp_cmd = string("/bin/cp ") + uprobes_home +
-    string("/Module.symvers ") + s.tmpdir;
+  vector<string> cp_cmd;
+  cp_cmd.push_back ("/bin/cp");
+  cp_cmd.push_back (s.runtime_path + "/uprobes/Module.symvers");
+  cp_cmd.push_back (s.tmpdir);
 
   return stap_system (s.verbose, cp_cmd);
 }
@@ -407,7 +435,8 @@ uprobes_pass (systemtap_session& s)
     return 0;
 
   if (s.kernel_config["CONFIG_UTRACE"] != string("y")) {
-    clog << "user-space facilities not available without kernel CONFIG_UTRACE" << endl;
+    clog << _("user-space facilities not available without kernel CONFIG_UTRACE") << endl;
+    s.set_try_server ();
     return 1;
   }
 
@@ -429,46 +458,72 @@ uprobes_pass (systemtap_session& s)
     rc = verify_uprobes_uptodate(s);
   if (rc == 0)
     rc = copy_uprobes_symbols(s);
+  if (rc)
+    s.set_try_server ();
   return rc;
 }
 
-string
+vector<string>
 make_run_command (systemtap_session& s, const string& module)
 {
   // for now, just spawn staprun
-  string staprun_cmd = string(getenv("SYSTEMTAP_STAPRUN") ?: BINDIR "/staprun")
-    + " "
-    + (s.verbose>1 ? "-v " : "")
-    + (s.verbose>2 ? "-v " : "")
-    + (s.suppress_warnings ? "-w " : "")
-    + (s.output_file.empty() ? "" : "-o " + s.output_file + " ");
+  vector<string> staprun_cmd;
+  staprun_cmd.push_back(getenv("SYSTEMTAP_STAPRUN") ?: BINDIR "/staprun");
+  if (s.verbose>1)
+    staprun_cmd.push_back("-v");
+  if (s.verbose>2)
+    staprun_cmd.push_back("-v");
+  if (s.suppress_warnings)
+    staprun_cmd.push_back("-w");
 
-  if (s.cmd != "")
-    staprun_cmd += "-c " + cmdstr_quoted(s.cmd) + " ";
+  if (!s.output_file.empty())
+    {
+      staprun_cmd.push_back("-o");
+      staprun_cmd.push_back(s.output_file);
+    }
+
+  if (!s.cmd.empty())
+    {
+      staprun_cmd.push_back("-c");
+      staprun_cmd.push_back(s.cmd);
+    }
 
   if (s.target_pid)
-    staprun_cmd += "-t " + lex_cast(s.target_pid) + " ";
+    {
+      staprun_cmd.push_back("-t");
+      staprun_cmd.push_back(lex_cast(s.target_pid));
+    }
 
   if (s.buffer_size)
-    staprun_cmd += "-b " + lex_cast(s.buffer_size) + " ";
+    {
+      staprun_cmd.push_back("-b");
+      staprun_cmd.push_back(lex_cast(s.buffer_size));
+    }
 
   if (s.need_uprobes)
-    staprun_cmd += "-u" + s.uprobes_path + " ";
+    {
+      staprun_cmd.push_back("-u");
+      if (!s.uprobes_path.empty())
+        staprun_cmd.back().append(s.uprobes_path);
+    }
 
   if (s.load_only)
-    staprun_cmd += (s.output_file.empty() ? "-L " : "-D ");
+    staprun_cmd.push_back(s.output_file.empty() ? "-L" : "-D");
 
   if (!s.size_option.empty())
-    staprun_cmd += "-S " + s.size_option + " ";
+    {
+      staprun_cmd.push_back("-S");
+      staprun_cmd.push_back(s.size_option);
+    }
 
   if (module.empty())
-    staprun_cmd += s.tmpdir + "/" + s.module_name + ".ko";
+    staprun_cmd.push_back(s.tmpdir + "/" + s.module_name + ".ko");
   else
-    staprun_cmd += module;
+    staprun_cmd.push_back(module);
 
   // add module arguments
-  for (unsigned i=0; i<s.globalopts.size(); i++)
-    staprun_cmd += " " + s.globalopts[i];
+  staprun_cmd.insert(staprun_cmd.end(),
+                     s.globalopts.begin(), s.globalopts.end());
 
   return staprun_cmd;
 }
@@ -487,7 +542,8 @@ make_tracequery(systemtap_session& s, string& name,
   if (create_dir(dir.c_str()) != 0)
     {
       if (! s.suppress_warnings)
-        cerr << "Warning: failed to create directory for querying tracepoints." << endl;
+        cerr << _("Warning: failed to create directory for querying tracepoints.") << endl;
+      s.set_try_server ();
       return 1;
     }
 
@@ -539,19 +595,26 @@ make_tracequery(systemtap_session& s, string& name,
   osrc.close();
 
   // make the module
-  string make_cmd = "make -C '" + s.kernel_build_tree + "'"
-    + " M='" + dir + "' modules";
+  vector<string> make_cmd;
+  make_cmd.push_back("make");
+  make_cmd.push_back("-C");
+  make_cmd.push_back(s.kernel_build_tree);
+  make_cmd.push_back("M=" + dir); // need make-quoting?
+  make_cmd.push_back("modules");
 
-  // Add architecture
-  make_cmd += string(" ARCH=") + lex_cast_qstring(s.architecture);
+  // Add architecture, except for old powerpc (RHBZ669082)
+  if (s.architecture != "powerpc" ||
+      (strverscmp (s.kernel_base_release.c_str(), "2.6.15") >= 0))
+    make_cmd.push_back("ARCH=" + s.architecture); // need make-quoting?
 
   // Add any custom kbuild flags
-  for (unsigned k=0; k<s.kbuildflags.size(); k++)
-    make_cmd += string(" ") + lex_cast_qstring(s.kbuildflags[k]);
+  make_cmd.insert(make_cmd.end(), s.kbuildflags.begin(), s.kbuildflags.end());
 
-  if (s.verbose < 4)
-    make_cmd += " >/dev/null 2>&1";
-  return run_make_cmd(s, make_cmd);
+  bool quiet = (s.verbose < 4);
+  int rc = run_make_cmd(s, make_cmd, quiet, quiet);
+  if (rc)
+    s.set_try_server ();
+  return rc;
 }
 
 
@@ -567,7 +630,8 @@ make_typequery_kmod(systemtap_session& s, const vector<string>& headers, string&
   if (create_dir(dir.c_str()) != 0)
     {
       if (! s.suppress_warnings)
-        cerr << "Warning: failed to create directory for querying types." << endl;
+        cerr << _("Warning: failed to create directory for querying types.") << endl;
+      s.set_try_server ();
       return 1;
     }
 
@@ -590,7 +654,7 @@ make_typequery_kmod(systemtap_session& s, const vector<string>& headers, string&
   //    @cast(foo, "bsd_acct_struct", "kernel<kernel/acct.c>")->...
   omf << "CFLAGS_" << basename << ".o :=";
   for (size_t i = 0; i < headers.size(); ++i)
-    omf << " -include " << lex_cast_qstring(headers[i]);
+    omf << " -include " << lex_cast_qstring(headers[i]); // XXX right quoting?
   omf << endl;
 
   omf << "obj-m := " + basename + ".o" << endl;
@@ -602,19 +666,26 @@ make_typequery_kmod(systemtap_session& s, const vector<string>& headers, string&
   osrc.close();
 
   // make the module
-  string make_cmd = "make -C '" + s.kernel_build_tree + "'"
-    + " M='" + dir + "' modules";
+  vector<string> make_cmd;
+  make_cmd.push_back("make");
+  make_cmd.push_back("-C");
+  make_cmd.push_back(s.kernel_build_tree);
+  make_cmd.push_back("M=" + dir); // need make-quoting?
+  make_cmd.push_back("modules");
 
-  // Add architecture
-  make_cmd += string(" ARCH=") + lex_cast_qstring(s.architecture);
+  // Add architecture, except for old powerpc (RHBZ669082)
+  if (s.architecture != "powerpc" ||
+      (strverscmp (s.kernel_base_release.c_str(), "2.6.15") >= 0))
+    make_cmd.push_back("ARCH=" + s.architecture); // need make-quoting?
 
   // Add any custom kbuild flags
-  for (unsigned k=0; k<s.kbuildflags.size(); k++)
-    make_cmd += string(" ") + lex_cast_qstring(s.kbuildflags[k]);
+  make_cmd.insert(make_cmd.end(), s.kbuildflags.begin(), s.kbuildflags.end());
 
-  if (s.verbose < 4)
-    make_cmd += " >/dev/null 2>&1";
-  return run_make_cmd(s, make_cmd);
+  bool quiet = (s.verbose < 4);
+  int rc = run_make_cmd(s, make_cmd, quiet, quiet);
+  if (rc)
+    s.set_try_server ();
+  return rc;
 }
 
 
@@ -632,13 +703,25 @@ make_typequery_umod(systemtap_session& s, const vector<string>& headers, string&
   // cwd in this case will be the cwd of stap itself though, which may be
   // trickier to deal with.  It might be better to "cd `dirname $script`"
   // first...
-  ostringstream cmd;
-  cmd << "gcc -shared -g -fno-eliminate-unused-debug-types -xc /dev/null -o " << name;
+  vector<string> cmd;
+  cmd.push_back("gcc");
+  cmd.push_back("-shared");
+  cmd.push_back("-g");
+  cmd.push_back("-fno-eliminate-unused-debug-types");
+  cmd.push_back("-xc");
+  cmd.push_back("/dev/null");
+  cmd.push_back("-o");
+  cmd.push_back(name);
   for (size_t i = 0; i < headers.size(); ++i)
-    cmd << " -include " << lex_cast_qstring(headers[i]);
-  if (s.verbose < 4)
-    cmd << " >/dev/null 2>&1";
-  return stap_system (s.verbose, cmd.str());
+    {
+      cmd.push_back("-include");
+      cmd.push_back(headers[i]);
+    }
+  bool quiet = (s.verbose < 4);
+  int rc = stap_system (s.verbose, cmd, quiet, quiet);
+  if (rc)
+    s.set_try_server ();
+  return rc;
 }
 
 
@@ -658,8 +741,15 @@ make_typequery(systemtap_session& s, string& module)
       if (end == string::npos)
         return -1;
       string header = module.substr(i, end - i);
-      assert_regexp_match("@cast header", header, "^[a-z0-9/_.+-]+$");
-      headers.push_back(header);
+      vector<string> matches;
+      if (regexp_match(header, "^[a-zA-Z0-9/_.+-]+$", matches))
+        {
+          if (! s.suppress_warnings)
+            cerr << _F("Warning: skipping malformed @cast header \"%s\"",
+                        header.c_str()) << endl;
+        }
+      else
+        headers.push_back(header);
     }
   if (headers.empty())
     return -1;
