@@ -12,8 +12,7 @@
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "config.h"
 #include "session.h"
@@ -128,7 +127,7 @@ void create_hash_log(const string &type_str, const string &parms, const string &
   log_file << "[" << time_str.substr(0,time_str.length()-1); // erase terminated '\n'
   log_file << "] " << type_str << ":" << endl;
   log_file << parms << endl;
-  log_file << "result:" << result << endl;
+  log_file << _("result:") << result << endl;
   log_file.close();
 }
 
@@ -154,13 +153,6 @@ get_base_hash (systemtap_session& s)
   h.add_path(s.kernel_build_tree + "/include/linux/version.h");
   h.add_path(s.kernel_build_tree + "/include/linux/utsrelease.h");
 
-  // If the kernel is a git working directory, then add the git HEAD
-  // revision to our hash as well.
-  // XXX avoiding this for now, because it's potentially expensive and has
-  // uncertain gain.  The only corner case that this may help is if a developer
-  // is switching the source tree without rebuilding the kernel...
-  ///h.add(git_revision(s.kernel_build_tree));
-
   // Hash runtime path (that gets added in as "-R path").
   h.add_path(s.runtime_path);
 
@@ -204,9 +196,8 @@ create_hashdir (systemtap_session& s, const string& result, string& hashdir)
       if (create_dir(hashdir.c_str()) != 0)
         {
           if (! s.suppress_warnings)
-            cerr << "Warning: failed to create cache directory (\""
-                 << hashdir + "\"): " << strerror(errno)
-                 << ", disabling cache support." << endl;
+            cerr << _F("Warning: failed to create cache directory (\"%s\") %s, disabling cache support",
+                       hashdir.c_str(), strerror(errno)) << endl;
 	  s.use_cache = s.use_script_cache = false;
 	  return false;
 	}
