@@ -1,5 +1,5 @@
 // systemtap translator/driver
-// Copyright (C) 2005-2010 Red Hat Inc.
+// Copyright (C) 2005-2011 Red Hat Inc.
 // Copyright (C) 2005 IBM Corp.
 // Copyright (C) 2006 Intel Corporation.
 //
@@ -24,6 +24,9 @@
 #include "csclient.h"
 #include "remote.h"
 
+#include <libintl.h>
+#include <locale.h>
+
 #include "stap-probe.h"
 
 #include <cstdlib>
@@ -173,7 +176,7 @@ printscript(systemtap_session& s, ostream& o)
   else
     {
       if (s.embeds.size() > 0)
-        o << "# global embedded code" << endl;
+        o << _("# global embedded code") << endl;
       for (unsigned i=0; i<s.embeds.size(); i++)
         {
           if (pending_interrupts) return;
@@ -183,7 +186,7 @@ printscript(systemtap_session& s, ostream& o)
         }
 
       if (s.globals.size() > 0)
-        o << "# globals" << endl;
+        o << _("# globals") << endl;
       for (unsigned i=0; i<s.globals.size(); i++)
         {
           if (pending_interrupts) return;
@@ -198,7 +201,7 @@ printscript(systemtap_session& s, ostream& o)
         }
 
       if (s.functions.size() > 0)
-        o << "# functions" << endl;
+        o << _("# functions") << endl;
       for (map<string,functiondecl*>::iterator it = s.functions.begin(); it != s.functions.end(); it++)
         {
           if (pending_interrupts) return;
@@ -206,7 +209,7 @@ printscript(systemtap_session& s, ostream& o)
           f->printsig (o);
           o << endl;
           if (f->locals.size() > 0)
-            o << "  # locals" << endl;
+            o << _("  # locals") << endl;
           for (unsigned j=0; j<f->locals.size(); j++)
             {
               vardecl* v = f->locals[j];
@@ -222,7 +225,7 @@ printscript(systemtap_session& s, ostream& o)
         }
 
       if (s.probes.size() > 0)
-        o << "# probes" << endl;
+        o << _("# probes") << endl;
       for (unsigned i=0; i<s.probes.size(); i++)
         {
           if (pending_interrupts) return;
@@ -230,7 +233,7 @@ printscript(systemtap_session& s, ostream& o)
           p->printsig (o);
           o << endl;
           if (p->locals.size() > 0)
-            o << "  # locals" << endl;
+            o << _("  # locals") << endl;
           for (unsigned j=0; j<p->locals.size(); j++)
             {
               vardecl* v = p->locals[j];
@@ -253,6 +256,8 @@ int pending_interrupts;
 extern "C"
 void handle_interrupt (int sig)
 {
+  // This might be nice, but we don't know our current verbosity...
+  // clog << _F("Received signal %d", sig) << endl << flush;
   kill_stap_spawn(sig);
   pending_interrupts ++;
   if (pending_interrupts > 1) // XXX: should be configurable? time-based?
@@ -270,6 +275,7 @@ setup_signals (sighandler_t handler)
 {
   struct sigaction sa;
 
+  memset(&sa, 0, sizeof(sa));
   sa.sa_handler = handler;
   sigemptyset (&sa.sa_mask);
   if (handler != SIG_IGN)
@@ -278,6 +284,8 @@ setup_signals (sighandler_t handler)
       sigaddset (&sa.sa_mask, SIGPIPE);
       sigaddset (&sa.sa_mask, SIGINT);
       sigaddset (&sa.sa_mask, SIGTERM);
+      sigaddset (&sa.sa_mask, SIGXFSZ);
+      sigaddset (&sa.sa_mask, SIGXCPU);
     }
   sa.sa_flags = SA_RESTART;
 
@@ -285,6 +293,8 @@ setup_signals (sighandler_t handler)
   sigaction (SIGPIPE, &sa, NULL);
   sigaction (SIGINT, &sa, NULL);
   sigaction (SIGTERM, &sa, NULL);
+  sigaction (SIGXFSZ, &sa, NULL);
+  sigaction (SIGXCPU, &sa, NULL);
 }
 
 int parse_kernel_config (systemtap_session &s)
@@ -294,8 +304,9 @@ int parse_kernel_config (systemtap_session &s)
   struct stat st;
   int rc = stat(kernel_config_file.c_str(), &st);
   if (rc != 0)
-    {  
-	clog << "Checking \"" << kernel_config_file << "\" failed: " << strerror(errno) << endl;
+    {
+        clog << _F("Checking \"%s\" failed with error: %s",
+                   kernel_config_file.c_str(), strerror(errno)) << endl;
 	find_devel_rpms(s, s.kernel_build_tree.c_str());
 	missing_rpm_list_print(s,"-devel");
 	return rc;
@@ -313,8 +324,10 @@ int parse_kernel_config (systemtap_session &s)
       s.kernel_config[key] = value;
     }
   if (s.verbose > 2)
-    clog << "Parsed kernel \"" << kernel_config_file << "\", number of tuples: " << s.kernel_config.size() << endl;
-  
+    clog << _F("Parsed kernel \"%s\", ", kernel_config_file.c_str())
+         << _F(ngettext("containing %zu tuple", "containing %zu tuples",
+                s.kernel_config.size()), s.kernel_config.size()) << endl;
+
   kcf.close();
   return 0;
 }
@@ -327,8 +340,8 @@ int parse_kernel_exports (systemtap_session &s)
   int rc = stat(kernel_exports_file.c_str(), &st);
   if (rc != 0)
     {
-	clog << "Checking \"" << kernel_exports_file << "\" failed: " << strerror(errno) << endl
-	     << "Ensure kernel development headers & makefiles are installed." << endl;
+        clog << _F("Checking \"%s\" failed with error: %s\nEnsure kernel development headers & makefiles are installed",
+                   kernel_exports_file.c_str(), strerror(errno)) << endl;
 	return rc;
     }
 
@@ -344,16 +357,22 @@ int parse_kernel_exports (systemtap_session &s)
         s.kernel_exports.insert (tokens[1]);
     }
   if (s.verbose > 2)
-    clog << "Parsed kernel \"" << kernel_exports_file << "\", number of vmlinux exports: " << s.kernel_exports.size() << endl;
-  
+    clog << _F(ngettext("Parsed kernel %s, which contained one vmlinux export",
+                        "Parsed kernel %s, which contained %zu vmlinux exports",
+                         s.kernel_exports.size()), kernel_exports_file.c_str(),
+                         s.kernel_exports.size()) << endl;
+
   kef.close();
   return 0;
 }
 
 
-static void
+static int
 create_temp_dir (systemtap_session &s)
 {
+  if (!s.tmpdir.empty())
+    return 0;
+
   // Create a temporary directory to build within.
   // Be careful with this, as "tmpdir" is "rm -rf"'d at the end.
   const char* tmpdir_env = getenv("TMPDIR");
@@ -368,47 +387,66 @@ create_temp_dir (systemtap_session &s)
   if (! tmpdir_name)
     {
       const char* e = strerror (errno);
-      cerr << "ERROR: cannot create temporary directory (\"" << tmpdirt << "\"): " << e << endl;
-      exit (1); // die
+      //TRANSLATORS: we can't make the directory due to the error
+      cerr << _F("ERROR: cannot create temporary directory (\" %s \"): %s", tmpdirt.c_str(), e) << endl;
+      return 1;
     }
   else
     s.tmpdir = tmpdir_name;
 
   if (s.verbose>1)
-    clog << "Created temporary directory \"" << s.tmpdir << "\"" << endl;
+    clog << _F("Created temporary directory \"%s\"", s.tmpdir.c_str()) << endl;
+  return 0;
 }
 
 static void
 remove_temp_dir (systemtap_session &s)
 {
-  if (s.tmpdir != "")
+  if (!s.tmpdir.empty())
     {
       if (s.keep_tmpdir)
         // NB: the format of this message needs to match the expectations
-        // of stap-server-connect.c.
-        clog << "Keeping temporary directory \"" << s.tmpdir << "\"" << endl;
+        // of stap-serverd.cxx.
+        clog << _F("Keeping temporary directory \"%s\"", s.tmpdir.c_str()) << endl;
       else
         {
-	  // Ignore signals while we're deleting the temporary directory.
-	  setup_signals (SIG_IGN);
+	  // Mask signals while we're deleting the temporary directory.
+	  stap_sigmasker masked;
 
 	  // Remove the temporary directory.
-          string cleanupcmd = "rm -rf ";
-          cleanupcmd += s.tmpdir;
+	  vector<string> cleanupcmd;
+	  cleanupcmd.push_back("rm");
+	  cleanupcmd.push_back("-rf");
+	  cleanupcmd.push_back(s.tmpdir);
 
 	  (void) stap_system (s.verbose, cleanupcmd);
+          s.tmpdir.clear();
         }
     }
 }
 
+// Compilation passes 0 through 4
 static int
 passes_0_4 (systemtap_session &s)
 {
   int rc = 0;
 
+  // If we don't know the release, there's no hope either locally or on a server.
+  if (s.kernel_release.empty())
+    {
+      if (s.kernel_build_tree.empty())
+        cerr << _("ERROR: kernel release isn't specified") << endl;
+      else
+        cerr << _F("ERROR: kernel release isn't found in \"%s\"",
+                   s.kernel_build_tree.c_str()) << endl;
+      return 1;
+    }
+
   // Create a temporary directory to build within.
   // Be careful with this, as "s.tmpdir" is "rm -rf"'d at the end.
-  create_temp_dir (s);
+  rc = create_temp_dir (s);
+  if (rc)
+    return rc;
 
   // Perform passes 0 through 4 using a compile server?
   if (! s.specified_servers.empty ())
@@ -417,7 +455,11 @@ passes_0_4 (systemtap_session &s)
       compile_server_client client (s);
       return client.passes_0_4 ();
 #else
-      cerr << "WARNING: Without NSS, using a compile-server is not supported by this version of systemtap" << endl;
+      cerr << _("WARNING: Without NSS, using a compile-server is not supported by this version of systemtap") << endl;
+      // This cannot be an attempt to use a server after a local compile failed
+      // since --use-server-on-error is locked to 'no' if we don't have
+      // NSS.
+      assert (! s.try_server ());
 #endif
     }
 
@@ -425,7 +467,6 @@ passes_0_4 (systemtap_session &s)
   s.verbose = s.perpass_verbose[0];
   PROBE1(stap, pass0__start, &s);
 
-
   // For PR1477, we used to override $PATH and $LC_ALL and other stuff
   // here.  We seem to use complete pathnames in
   // buildrun.cxx/tapsets.cxx now, so this is not necessary.  Further,
@@ -434,31 +475,25 @@ passes_0_4 (systemtap_session &s)
 
   s.kernel_base_release.assign(s.kernel_release, 0, s.kernel_release.find('-'));
 
-  // arguments parsed; get down to business
-  if (s.verbose > 1)
+  // Now that no further changes to s.kernel_build_tree can occur, let's use it.
+  if ((rc = parse_kernel_config (s)) != 0)
     {
-      s.version ();
-      clog << "Session arch: " << s.architecture
-           << " release: " << s.kernel_release
-           << endl;
+      // Try again with a server
+      s.set_try_server ();
+      return rc;
     }
 
-  // Now that no further changes to s.kernel_build_tree can occur, let's use it.
-  if (parse_kernel_config (s) != 0)
-    exit (1);
-
-  if (parse_kernel_exports (s) != 0)
-    exit (1);
-
+  if ((rc = parse_kernel_exports (s)) != 0)
+    {
+      // Try again with a server
+      s.set_try_server ();
+      return rc;
+    }
 
   // Create the name of the C source file within the temporary
   // directory.
   s.translated_source = string(s.tmpdir) + "/" + s.module_name + ".c";
 
-  // Set up our handler to catch routine signals, to allow clean
-  // and reasonably timely exit.
-  setup_signals(&handle_interrupt);
-
   PROBE1(stap, pass0__end, &s);
 
   struct tms tms_before;
@@ -488,8 +523,10 @@ passes_0_4 (systemtap_session &s)
       s.user_file = parse (s, ii, s.guru_mode);
     }
   if (s.user_file == 0)
-    // syntax errors already printed
-    rc ++;
+    {
+      // Syntax errors already printed.
+      rc ++;
+    }
 
   // Construct arch / kernel-versioning search path
   vector<string> version_suffixes;
@@ -532,8 +569,8 @@ passes_0_4 (systemtap_session &s)
           string dir = s.include_path[i] + version_suffixes[k] + "/*.stp";
           int r = glob(dir.c_str (), 0, NULL, & globbuf);
           if (r == GLOB_NOSPACE || r == GLOB_ABORTED)
-            rc ++;
-          // GLOB_NOMATCH is acceptable
+	    rc ++;
+	  // GLOB_NOMATCH is acceptable
 
           unsigned prev_s_library_files = s.library_files.size();
 
@@ -548,8 +585,9 @@ passes_0_4 (systemtap_session &s)
                   user_file_stat.st_dev == tapset_file_stat.st_dev &&
                   user_file_stat.st_ino == tapset_file_stat.st_ino)
                 {
-                  cerr << "usage error: tapset file '" << globbuf.gl_pathv[j]
-                       << "' cannot be run directly as a session script." << endl;
+                  cerr 
+                  << _F("usage error: tapset file '%s' cannot be run directly as a session script.",
+                        globbuf.gl_pathv[j]) << endl;
                   rc ++;
                 }
 
@@ -574,9 +612,10 @@ passes_0_4 (systemtap_session &s)
 
           unsigned next_s_library_files = s.library_files.size();
           if (s.verbose>1 && globbuf.gl_pathc > 0)
-            clog << "Searched \"" << dir << "\","
-                 << " found " << globbuf.gl_pathc 
-                 << " processed " << (next_s_library_files-prev_s_library_files) << endl;
+            //TRANSLATORS: Searching through directories, 'processed' means 'examined so far'
+            clog << _F("Searched: \" %s \", found: %zu, processed: %u",
+                       dir.c_str(), globbuf.gl_pathc,
+                       (next_s_library_files-prev_s_library_files)) << endl;
 
           globfree (& globbuf);
         }
@@ -586,7 +625,7 @@ passes_0_4 (systemtap_session &s)
 
   if (rc == 0 && s.last_pass == 1)
     {
-      cout << "# parse tree dump" << endl;
+      cout << _("# parse tree dump") << endl;
       s.user_file->print (cout);
       cout << endl;
       if (s.verbose)
@@ -623,9 +662,10 @@ passes_0_4 (systemtap_session &s)
     }
 
   if (rc && !s.listing_mode)
-    cerr << "Pass 1: parse failed.  "
-         << "Try again with another '--vp 1' option."
-         << endl;
+    cerr << _("Pass 1: parse failed.  Try again with another '--vp 1' option.") << endl;
+    //cerr << "Pass 1: parse failed.  "
+    //     << "Try again with another '--vp 1' option."
+    //     << endl;
 
   PROBE1(stap, pass1__end, &s);
 
@@ -654,10 +694,11 @@ passes_0_4 (systemtap_session &s)
                       << TIMESPRINT
                       << endl;
 
-  if (rc && !s.listing_mode)
-    cerr << "Pass 2: analysis failed.  "
-         << "Try again with another '--vp 01' option."
-         << endl;
+  if (rc && !s.listing_mode && !s.try_server ())
+    cerr << _("Pass 2: analysis failed.  Try again with another '--vp 01' option.") << endl;
+    //cerr << "Pass 2: analysis failed.  "
+    //     << "Try again with another '--vp 01' option."
+    //     << endl;
 
   /* Print out list of missing files.  XXX should be "if (rc)" ? */
   missing_rpm_list_print(s,"-debuginfo");
@@ -707,8 +748,7 @@ passes_0_4 (systemtap_session &s)
   PROBE1(stap, pass3__start, &s);
 
   rc = translate_pass (s);
-
-  if (rc == 0 && s.last_pass == 3)
+  if (! rc && s.last_pass == 3)
     {
       ifstream i (s.translated_source.c_str());
       cout << i.rdbuf();
@@ -717,18 +757,20 @@ passes_0_4 (systemtap_session &s)
   times (& tms_after);
   gettimeofday (&tv_after, NULL);
 
-  if (s.verbose) clog << "Pass 3: translated to C into \""
-                      << s.translated_source
-                      << "\" "
-                      << getmemusage()
-                      << TIMESPRINT
-                      << endl;
-
-  if (rc)
-    cerr << "Pass 3: translation failed.  "
-         << "Try again with another '--vp 001' option."
+  if (s.verbose) 
+    clog << "Pass 3: translated to C into \""
+         << s.translated_source
+         << "\" "
+         << getmemusage()
+         << TIMESPRINT
          << endl;
 
+  if (rc && ! s.try_server ())
+    cerr << _("Pass 3: translation failed.  Try again with another '--vp 001' option.") << endl;
+    //cerr << "Pass 3: translation failed.  "
+    //     << "Try again with another '--vp 001' option."
+    //     << endl;
+
   PROBE1(stap, pass3__end, &s);
 
   if (rc || s.last_pass == 3 || pending_interrupts) return rc;
@@ -745,8 +787,7 @@ passes_0_4 (systemtap_session &s)
       get_stapconf_from_cache(s);
     }
   rc = compile_pass (s);
-
-  if (rc == 0 && s.last_pass == 4)
+  if (! rc && s.last_pass == 4)
     {
       cout << ((s.hash_path == "") ? (s.module_name + string(".ko")) : s.hash_path);
       cout << endl;
@@ -761,10 +802,11 @@ passes_0_4 (systemtap_session &s)
                       << TIMESPRINT
                       << endl;
 
-  if (rc)
-    cerr << "Pass 4: compilation failed.  "
-         << "Try again with another '--vp 0001' option."
-         << endl;
+  if (rc && ! s.try_server ())
+    cerr << _("Pass 4: compilation failed.  Try again with another '--vp 0001' option.") << endl;
+    //cerr << "Pass 4: compilation failed.  "
+    //     << "Try again with another '--vp 0001' option."
+    //     << endl;
   else
     {
       // Update cache. Cache cleaning is kicked off at the beginning of this function.
@@ -805,12 +847,8 @@ pass_5 (systemtap_session &s, vector<remote*> targets)
   // a "hello, I'm starting" message, but then the others aren't interactive
   // and don't take an indefinite amount of time.
   PROBE1(stap, pass5__start, &s);
-  if (s.verbose) clog << "Pass 5: starting run." << endl;
-  int rc = 0; // XXX with multiple targets, need to deal with partial failure
-  for (unsigned i = 0; i < targets.size() && !pending_interrupts; ++i)
-    rc |= targets[i]->start();
-  for (unsigned i = 0; i < targets.size(); ++i)
-    rc |= targets[i]->finish();
+  if (s.verbose) clog << _("Pass 5: starting run.") << endl;
+  int rc = remote::run(targets);
   struct tms tms_after;
   times (& tms_after);
   unsigned _sc_clk_tck = sysconf (_SC_CLK_TCK);
@@ -821,9 +859,10 @@ pass_5 (systemtap_session &s, vector<remote*> targets)
                       << endl;
 
   if (rc)
-    cerr << "Pass 5: run failed.  "
-         << "Try again with another '--vp 00001' option."
-         << endl;
+    cerr << _("Pass 5: run failed.  Try again with another '--vp 00001' option.") << endl;
+    //cerr << "Pass 5: run failed.  "
+    //     << "Try again with another '--vp 00001' option."
+    //     << endl;
   else
     // Interrupting pass-5 to quit is normal, so we want an EXIT_SUCCESS below.
     pending_interrupts = 0;
@@ -848,7 +887,7 @@ cleanup (systemtap_session &s, int rc)
 #ifdef HAVE_LIBSQLITE3
     update_coverage_db(s);
 #else
-    cerr << "Coverage database not available without libsqlite3" << endl;
+    cerr << _("Coverage database not available without libsqlite3") << endl;
 #endif
   }
 
@@ -858,12 +897,40 @@ cleanup (systemtap_session &s, int rc)
   PROBE1(stap, pass6__end, &s);
 }
 
+static int
+passes_0_4_again_with_server (systemtap_session &s)
+{
+  // Not a server and not already using a server.
+  assert (! s.client_options);
+  assert (s.specified_servers.empty ());
+
+  // Specify default server(s).
+  s.specified_servers.push_back ("");
+
+  // Remove the previous temporary directory and start fresh.
+  remove_temp_dir (s);
+
+  // Try to compile again, using the server
+  clog << _("Attempting compilation using a compile server")
+       << endl;
+  int rc = passes_0_4 (s);
+  return rc;
+}
+
 int
 main (int argc, char * const argv [])
 {
   // Initialize defaults.
   systemtap_session s;
 
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
+  // Set up our handler to catch routine signals, to allow clean
+  // and reasonably timely exit.
+  setup_signals(&handle_interrupt);
+
   // Process the command line.
   int rc = s.parse_cmdline (argc, argv);
   if (rc != 0)
@@ -872,60 +939,69 @@ main (int argc, char * const argv [])
   // Check for options conflicts. Exits if errors are detected.
   s.check_options (argc, argv);
 
-  // If requested, query server status. This is independent of other tasks.
-  query_server_status (s);
+  // arguments parsed; get down to business
+  if (s.verbose > 1)
+    s.version ();
 
-  // If requested, manage trust of servers. This is independent of other tasks.
-  manage_server_trust (s);
+  // Some of the remote methods need to write temporary data, so go ahead
+  // and create the main tempdir now.
+  rc = create_temp_dir (s);
 
-  // Run the passes only if a script has been specified. The requirement for
-  // a script has already been checked in systemtap_session::check_options.
-  if (s.have_script)
+  // Prepare connections for each specified remote target.
+  vector<remote*> targets;
+  if (s.remote_uris.empty())
+    s.remote_uris.push_back("direct:");
+  for (unsigned i = 0; rc == 0 && i < s.remote_uris.size(); ++i)
     {
-      vector<remote*> targets;
-      if (s.remote_uris.empty())
-	{
-	  remote* target = remote::create(s, "direct");
-	  if (target)
-	    targets.push_back(target);
-	  else
-	    rc = 1;
-	}
+      remote *target = remote::create(s, s.remote_uris[i]);
+      if (target)
+        targets.push_back(target);
       else
-	for (unsigned i = 0; i < s.remote_uris.size(); ++i)
-	  {
-	    remote *target = remote::create(s, s.remote_uris[i]);
-	    if (target)
-	      targets.push_back(target);
-	    else
-	      {
-		rc = 1;
-		break;
-	      }
-	  }
-
-      // Run passes 0-4 for each unique session,
-      // either locally or using a compile-server.
-      if (rc == 0)
-	{
-	  set<systemtap_session*> sessions;
-	  for (unsigned i = 0; i < targets.size(); ++i)
-	    sessions.insert(targets[i]->get_session());
-	  for (set<systemtap_session*>::iterator it = sessions.begin();
-	       it != sessions.end(); ++it)
-	    if ((rc = passes_0_4 (**it)))
-	      break;
-	}
-
-      // Run pass 5, if requested
-      if (rc == 0 && s.last_pass >= 5 && ! pending_interrupts)
-	rc = pass_5 (s, targets);
-
-      for (unsigned i = 0; i < targets.size(); ++i)
-	delete targets[i];
+        rc = 1;
     }
 
+  // Discover and loop over each unique session created by the remote targets.
+  set<systemtap_session*> sessions;
+  for (unsigned i = 0; i < targets.size(); ++i)
+    sessions.insert(targets[i]->get_session());
+  for (set<systemtap_session*>::iterator it = sessions.begin();
+       rc == 0 && !pending_interrupts && it != sessions.end(); ++it)
+    {
+      systemtap_session& ss = **it;
+      if (ss.verbose > 1)
+        clog << _F("Session arch: %s release: %s",
+                   ss.architecture.c_str(), ss.kernel_release.c_str()) << endl;
+
+      // If requested, query server status. This is independent of other tasks.
+      query_server_status (ss);
+
+      // If requested, manage trust of servers. This is independent of other tasks.
+      manage_server_trust (ss);
+
+      // Run the passes only if a script has been specified. The requirement for
+      // a script has already been checked in systemtap_session::check_options.
+      if (ss.have_script)
+        {
+          // Run passes 0-4 for each unique session,
+          // either locally or using a compile-server.
+          ss.init_try_server ();
+          if ((rc = passes_0_4 (ss)))
+            {
+              // Compilation failed.
+              // Try again using a server if appropriate.
+              if (ss.try_server ())
+                rc = passes_0_4_again_with_server (ss);
+            }
+        }
+    }
+
+  // Run pass 5, if requested
+  if (rc == 0 && s.have_script && s.last_pass >= 5 && ! pending_interrupts)
+    rc = pass_5 (s, targets);
+
   // Pass 6. Cleanup
+  for (unsigned i = 0; i < targets.size(); ++i)
+    delete targets[i];
   cleanup (s, rc);
 
   return (rc||pending_interrupts) ? EXIT_FAILURE : EXIT_SUCCESS;
