@@ -67,7 +67,7 @@ class Bench
       load
       sum=0
       `./itest #{threads} > #{@dir}/bench`
-      `sudo killall -HUP staprun`
+      `sudo killall -HUP stapio`
       Process.wait	# wait for staprun to exit
       @results[threads] = `cat #{@dir}/bench`.split[0].to_i - @@ftime[threads]
       File.open("#{@dir}/xxx.out") do |file|
@@ -129,7 +129,7 @@ class Bench
   @@minfreq = 0
 
   def cleanup
-    system('sudo killall -HUP staprun &> /dev/null')
+    system('sudo killall -HUP stapio &> /dev/null')
     system('sudo /sbin/rmmod bench &> /dev/null')
     `/bin/rm -f stap.out` if File.exists?("stap.out")
     `/bin/rm -f bench.stp` if File.exists?("bench.stp")
@@ -138,9 +138,8 @@ class Bench
   end
 
   def load
-    args = "-q"
-    fork do exec "sudo #{@@staprun} #{args} -o #{@outfile} #{@dir}/bench.ko &> #{@dir}/xxx.out" end
-    sleep 10
+    fork do exec "sudo #{@@staprun} -o #{@outfile} #{@dir}/bench.ko &> #{@dir}/xxx.out" end
+    sleep 5
   end
 
   def compile
@@ -264,8 +263,12 @@ class Stapbench < Bench
       load
       sum=0
       `./itest #{threads} > bench`
-      `sudo killall -HUP staprun`
-      Process.wait	# wait for staprun to exit
+      `sudo killall -HUP stapio`
+      if ($? != 0)
+	puts "status=#{$?}"
+	system("tail xxx.out")
+      end
+      Process.wait	# wait for stapio to exit
       @results[threads] = `cat bench`.split[0].to_i - @@ftime[threads]
       File.open("xxx.out") do |file|
 	file.each_line do |line|
@@ -283,10 +286,8 @@ class Stapbench < Bench
   protected
 
   def load
-    args = "-vv"
-    if @trans == BULK then args = "-bvv" end
-    fork do exec "stap #{args} -o #{@outfile} bench.stp &> xxx.out" end
-    sleep 10
+    fork do exec "sudo #{@@staprun} bench.ko &> xxx.out" end
+    sleep 5
   end
   
   def compile
@@ -312,6 +313,14 @@ class Stapbench < Bench
     else
       puts "NO CODE!"
     end
+    args = "-p4 -vv -DSTP_NO_OVERLOAD"
+    if @trans == BULK then args = "-p4 -bvv -DSTP_NO_OVERLOAD" end
+    `stap #{args} -m bench.ko bench.stp &> xxx.out`
+    if ($? != 0)
+      puts "compile failed. status=#{$?}"
+      system("tail xxx.out")
+      exit
+    end
   end
-
+  
 end
