@@ -24,6 +24,7 @@
 int ncpus;
 static int use_old_transport = 0;
 static int pending_interrupts = 0;
+static int target_pid_failed_p = 0;
 
 //enum _stp_sig_type { sig_none, sig_done, sig_detach };
 //static enum _stp_sig_type got_signal = sig_none;
@@ -54,12 +55,29 @@ static void *signal_thread(void *arg)
 static void chld_proc(int signum)
 {
   int32_t rc, btype = STP_EXIT;
+  int chld_stat = 0;
   dbug(2, "chld_proc %d (%s)\n", signum, strsignal(signum));
-  pid_t pid = waitpid(-1, NULL, WNOHANG);
-  if (pid != target_pid)
+  pid_t pid = waitpid(-1, &chld_stat, WNOHANG);
+  if (pid != target_pid) {
     return;
-  // send STP_EXIT
-  rc = write(control_channel, &btype, sizeof(btype));
+  }
+
+  if (chld_stat) {
+    // our child exited with a non-zero status
+    if (WIFSIGNALED(chld_stat)) {
+      err(_("Warning: child process exited with signal %d (%s)\n"),
+          WTERMSIG(chld_stat), strsignal(WTERMSIG(chld_stat)));
+      target_pid_failed_p = 1;
+    }
+    if (WIFEXITED(chld_stat) && WEXITSTATUS(chld_stat)) {
+      err(_("Warning: child process exited with status %d\n"),
+          WEXITSTATUS(chld_stat));
+      target_pid_failed_p = 1;
+    }
+  }
+
+  rc = write(control_channel, &btype, sizeof(btype)); // send STP_EXIT
+  (void) rc; /* XXX: notused */
 }
 
 #if WORKAROUND_BZ467568
@@ -103,7 +121,7 @@ static void setup_main_signals(void)
   sigaddset(s, SIGQUIT);
   pthread_sigmask(SIG_SETMASK, s, NULL);
   if (pthread_create(&tid, NULL, signal_thread, s) < 0) {
-    _perr("failed to create thread");
+    _perr(_("failed to create thread"));
     exit(1);
   }
 }
@@ -128,6 +146,7 @@ void start_cmd(void)
 
   /* if we are execing a target cmd, ignore ^C in stapio */
   /* and let the target cmd get it. */
+  memset(&a, 0, sizeof(a));
   sigemptyset(&a.sa_mask);
   a.sa_flags = 0;
   a.sa_handler = SIG_IGN;
@@ -139,6 +158,7 @@ void start_cmd(void)
   sigaddset (&blockmask, SIGUSR1);
 
   /* Establish the SIGUSR1 signal handler. */
+  memset(&usr1_action, 0, sizeof(usr1_action));
   sigfillset (&usr1_action.sa_mask);
   usr1_action.sa_flags = 0;
   usr1_action.sa_handler = signal_usr1;
@@ -183,10 +203,10 @@ void start_cmd(void)
           case 0:
             break;
           case WRDE_SYNTAX:
-            _err ("wordexp: syntax error (unmatched quotes?) in -c COMMAND\n");
+            _err (_("wordexp: syntax error (unmatched quotes?) in -c COMMAND\n"));
             _exit(1);
           default:
-            _err ("wordexp: parsing error (%d)\n", rc);
+            _err (_("wordexp: parsing error (%d)\n"), rc);
             _exit (1);
           }
         if (words.we_wordc < 1) { _err ("empty -c COMMAND"); _exit (1); }
@@ -294,13 +314,13 @@ static void read_buffer_info(void)
 
   len = read(fd, buf, sizeof(buf));
   if (len <= 0) {
-    perr("Couldn't read bufsize");
+    perr(_("Couldn't read bufsize"));
     close(fd);
     return;
   }
   ret = sscanf(buf, "%u,%u", &n_subbufs, &subbuf_size);
   if (ret != 2)
-    perr("Couldn't read bufsize");
+    perr(_("Couldn't read bufsize"));
 
   dbug(2, "n_subbufs= %u, size=%u\n", n_subbufs, subbuf_size);
   close(fd);
@@ -320,7 +340,7 @@ int init_stapio(void)
   /* create control channel */
   use_old_transport = init_ctl_channel(modname, 1);
   if (use_old_transport < 0) {
-    err("Failed to initialize control channel.\n");
+    err(_("Failed to initialize control channel.\n"));
     return -1;
   }
   read_buffer_info();
@@ -355,7 +375,7 @@ int init_stapio(void)
     /* daemonize */
     ret = daemon(0, 1); /* don't close stdout at this time. */
     if (ret) {
-      err("Failed to daemonize stapio\n");
+      err(_("Failed to daemonize stapio\n"));
       return -1;
     }
 
@@ -370,7 +390,7 @@ int init_stapio(void)
     /* redirect all outputs to /dev/null */
     ret = open("/dev/null", O_RDWR);
     if (ret < 0) {
-      err("Failed to open /dev/null\n");
+      err(_("Failed to open /dev/null\n"));
       return -1;
     }
     close(STDIN_FILENO);
@@ -421,7 +441,7 @@ void cleanup_and_exit(int detach, int rc)
   close_ctl_channel();
 
   if (detach) {
-    err("\nDisconnecting from systemtap module.\n" "To reconnect, type \"staprun -A %s\"\n", modname);
+    err(_("\nDisconnecting from systemtap module.\n" "To reconnect, type \"staprun -A %s\"\n"), modname);
     _exit(0);
   }
 
@@ -434,6 +454,7 @@ void cleanup_and_exit(int detach, int rc)
 
   // So that waitpid() below will work correctly, we need to clear
   // out our SIGCHLD handler.
+  memset(&sa, 0, sizeof(sa));
   sigemptyset(&sa.sa_mask);
   sa.sa_flags = 0;
   sa.sa_handler = SIG_DFL;
@@ -470,8 +491,12 @@ void cleanup_and_exit(int detach, int rc)
   }
 
   if (WIFEXITED(rstatus)) {
-          _exit(rc ?: WEXITSTATUS(rstatus));
+          if(rc || target_pid_failed_p || rstatus) // if we have an error
+            _exit(1);
+          else
+            _exit(0); //success
   }
+
   _exit(-1);
 }
 
@@ -530,7 +555,7 @@ int stp_main_loop(void)
     dbug(3, "nb=%ld\n", (long)nb);
     if (nb < (ssize_t) sizeof(recvbuf.type)) {
       if (nb >= 0 || (errno != EINTR && errno != EAGAIN)) {
-        _perr("Unexpected EOF in read (nb=%ld)", (long)nb);
+        _perr(_("Unexpected EOF in read (nb=%ld)"), (long)nb);
         cleanup_and_exit(0, 1);
       }
       dbug(4, "sleeping\n");
@@ -545,13 +570,13 @@ int stp_main_loop(void)
 #if STP_TRANSPORT_VERSION == 1
     case STP_REALTIME_DATA:
       if (write_realtime_data(recvbuf.payload.data, nb)) {
-        _perr("write error (nb=%ld)", (long)nb);
+        _perr(_("write error (nb=%ld)"), (long)nb);
         cleanup_and_exit(0, 1);
       }
       break;
 #endif
     case STP_OOB_DATA:
-      if (strncmp(recvbuf.payload.data, "WARNING:", 7) == 0) {
+      if (strncmp(recvbuf.payload.data, _("WARNING:"), 7) == 0) {
               if (suppress_warnings) break;
               if (verbose) { /* don't eliminate duplicates */
                       eprintf("%.*s", (int) nb, recvbuf.payload.data);
@@ -578,7 +603,7 @@ int stp_main_loop(void)
                                  overflow staprun's memory. */
 #define MAX_STORED_WARNINGS 1024
                               if (seen_count++ == MAX_STORED_WARNINGS) {
-                                      eprintf("WARNING deduplication table full\n");
+                                      eprintf(_("WARNING deduplication table full\n"));
                                       free (dupstr);
                               }
                               else if (seen_count > MAX_STORED_WARNINGS) {
@@ -622,6 +647,7 @@ int stp_main_loop(void)
         dbug(2, "got STP_REQUEST_EXIT\n");
         int32_t rc, btype = STP_EXIT;
         rc = write(control_channel, &btype, sizeof(btype));
+        (void) rc; /* XXX: notused */
         break;
       }
     case STP_START:
@@ -643,7 +669,7 @@ int stp_main_loop(void)
           int rc = ptrace (PTRACE_DETACH, target_pid, 0, 0);
           if (rc < 0)
             {
-              perror ("ptrace detach");
+              perror (_("ptrace detach"));
               if (target_cmd)
                 kill(target_pid, SIGKILL);
               cleanup_and_exit(0, 1);
@@ -676,7 +702,7 @@ int stp_main_loop(void)
         break;
       }
     default:
-      err("WARNING: ignored message of type %d\n", recvbuf.type);
+      err(_("WARNING: ignored message of type %d\n"), recvbuf.type);
     }
   }
   fclose(ofp);
