@@ -122,7 +122,7 @@ be_derived_probe_group::emit_module_decls (systemtap_session& s)
   sort(probes.begin(), probes.end(), be_derived_probe::comp);
 
   s.op->newline() << "static struct stap_be_probe {";
-  s.op->newline(1) << "struct stap_probe probe;";
+  s.op->newline(1) << "struct stap_probe * const probe;";
   s.op->newline() << "int state, type;";
   s.op->newline(-1) << "} stap_be_probes[] = {";
   s.op->indent(1);
@@ -140,7 +140,7 @@ be_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->newline() << "static void enter_be_probe (struct stap_be_probe *stp) {";
   s.op->indent(1);
   common_probe_entryfn_prologue (s.op, "stp->state", "stp->probe", false);
-  s.op->newline() << "(*stp->probe.ph) (c);";
+  s.op->newline() << "(*stp->probe->ph) (c);";
   common_probe_entryfn_epilogue (s.op, false);
   s.op->newline(-1) << "}";
 }
@@ -188,7 +188,6 @@ be_derived_probe_group::emit_module_exit (systemtap_session& s)
 
 struct never_derived_probe: public derived_probe
 {
-  never_derived_probe (probe* p): derived_probe (p) {}
   never_derived_probe (probe* p, probe_point* l): derived_probe (p, l) {}
   void join_group (systemtap_session&) { /* thus no probe_group */ }
   void emit_unprivileged_assertion (translator_output*) {}
