@@ -76,7 +76,7 @@ itrace_derived_probe::itrace_derived_probe (systemtap_session &s,
   derived_probe(p, l), has_path(hp), path(pn), pid(pd), single_step(ss)
 {
   if (s.kernel_config["CONFIG_UTRACE"] != string("y"))
-    throw semantic_error ("process probes not available without kernel CONFIG_UTRACE");
+    throw semantic_error (_("process probes not available without kernel CONFIG_UTRACE"));
 }
 
 
