@@ -97,10 +97,10 @@ mark_var_expanding_visitor::visit_target_symbol_arg (target_symbol* e)
   int argnum = atoi (argnum_s.c_str());
 
   if (argnum < 1 || argnum > (int)mark_args.size())
-    throw semantic_error ("invalid marker argument number", e->tok);
+    throw semantic_error (_("invalid marker argument number"), e->tok);
 
   if (is_active_lvalue (e))
-    throw semantic_error("write to marker parameter not permitted", e->tok);
+    throw semantic_error(_("write to marker parameter not permitted"), e->tok);
 
   e->assert_no_components("marker");
 
@@ -120,7 +120,7 @@ mark_var_expanding_visitor::visit_target_symbol_context (target_symbol* e)
   string sname = e->name;
 
   if (is_active_lvalue (e))
-    throw semantic_error("write to marker '" + sname + "' not permitted", e->tok);
+    throw semantic_error(_F("write to marker '%s' not permitted", sname.c_str()), e->tok);
 
   e->assert_no_components("marker");
 
@@ -184,7 +184,7 @@ mark_var_expanding_visitor::visit_target_symbol (target_symbol* e)
   try
     {
       if (e->addressof)
-        throw semantic_error("cannot take address of marker variable", e->tok);
+        throw semantic_error(_("cannot take address of marker variable"), e->tok);
 
       if (startswith(e->name, "$arg"))
         visit_target_symbol_arg (e);
@@ -192,7 +192,7 @@ mark_var_expanding_visitor::visit_target_symbol (target_symbol* e)
                || e->name == "$$parms" || e->name == "$$vars")
         visit_target_symbol_context (e);
       else
-        throw semantic_error ("invalid target symbol for marker, $argN, $name, $format, $$parms or $$vars expected",
+        throw semantic_error (_("invalid target symbol for marker, $argN, $name, $format, $$parms or $$vars expected"),
                               e->tok);
     }
   catch (const semantic_error &er)
@@ -238,8 +238,8 @@ mark_derived_probe::mark_derived_probe (systemtap_session &s,
       }
 
   if (sess.verbose > 2)
-    clog << "marker-based " << name << " mark=" << probe_name
-	 << " fmt='" << probe_format << "'" << endl;
+    clog << "marker-based " << name << " mark=" << probe_name << " fmt='" << probe_format
+         << "'" << endl;
 }
 
 
@@ -446,7 +446,7 @@ mark_derived_probe::initialize_probe_context_vars (translator_output* o)
 	  break;
 
 	default:
-	  throw semantic_error ("cannot expand unknown type");
+	  throw semantic_error (_("cannot expand unknown type"));
 	  break;
 	}
     }
@@ -579,7 +579,7 @@ public:
     if (! mark_cache.empty())
       {
         if (s.verbose > 3)
-          clog << "mark_builder releasing cache" << endl;
+          clog << _("mark_builder releasing cache") << endl;
 	mark_cache.clear();
       }
   }
@@ -616,7 +616,8 @@ mark_builder::build(systemtap_session & sess,
       if (! module_markers)
         {
 	  if (sess.verbose>3)
-	    clog << module_markers_path << " cannot be opened: "
+            //TRANSLATORS: specific path cannot be opened
+	    clog << module_markers_path << _(" cannot be opened: ")
 		 << strerror(errno) << endl;
 	  return;
 	}
